package net.wanji.utc.hisense.netty;


import cn.hutool.core.util.HexUtil;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.Unpooled;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.DatagramPacket;
import io.netty.channel.socket.nio.NioDatagramChannel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.hisense.cache.CrossInfoCache;
import net.wanji.utc.hisense.cache.netty.NettyMessageCache;
import net.wanji.utc.hisense.netty.codec.MessageDecoder;
import net.wanji.utc.hisense.netty.codec.MessageEnCoder;
import net.wanji.utc.hisense.netty.handler.UdpClientHandler;
import net.wanji.utc.hisense.pojo.netty.MessageResultPojo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.net.InetSocketAddress;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

@Slf4j
@Component
public class UdpClient {
    private static ChannelFuture udpChannelFuture = null;
    @Resource(name = "commonThreadPoolExecutor")
    private ThreadPoolTaskExecutor commonThreadPoolExecutor;

    /**
     * 建立连接
     */
    public static void connection(int localPort, int remotePort) throws InterruptedException {
        EventLoopGroup group = new NioEventLoopGroup();
        Bootstrap bootstrap = new Bootstrap();
        bootstrap.group(group)
                .channel(NioDatagramChannel.class)
                .option(ChannelOption.RCVBUF_ALLOCATOR, new FixedRecvByteBufAllocator(65535))
                .localAddress(new InetSocketAddress(localPort))
//                .remoteAddress(new InetSocketAddress(remotePort))
                .handler(new ChannelInitializer<NioDatagramChannel>() {
                    @Override
                    protected void initChannel(NioDatagramChannel datagramChannel) {
                        datagramChannel.pipeline()
                                .addLast("decoder", new MessageDecoder())
                                .addLast("encoder", new MessageEnCoder())
                                .addLast(new UdpClientHandler());
                    }
                });
        ChannelFuture channelFuture = bootstrap.bind().sync();
        Runtime.getRuntime().addShutdownHook(new Thread(() -> {
            group.shutdownGracefully();
            channelFuture.channel().closeFuture().syncUninterruptibly();
            log.warn("udp服务关闭！");
        }));
        log.info("udp服务正在运行，端口：{}", localPort);
        UdpClient.udpChannelFuture = channelFuture;
    }

    public static void sendMessage(String ip, Integer port, String msg) {
        byte[] bytes = HexUtil.decodeHex(msg);
        try {
            if (udpChannelFuture == null) {
                return;
            }
            udpChannelFuture.channel()
                    .writeAndFlush(new DatagramPacket(Unpooled.copiedBuffer(bytes), new InetSocketAddress(ip, port))).sync();
            log.debug("发送信号机:{}/{}命令消息:{}", ip, port, msg);
        } catch (InterruptedException e) {
            log.error("sendMsg is error", e);
        }
    }

    public static MessageResultPojo sendMessage(String ip, Integer port, String msg, String command, int timeout) {
        try {
            if (timeout > 0) {
                CountDownLatch countDownLatch = new CountDownLatch(1);
                String key = StringUtils.join("/", ip, ":", port, "/", command);
                long now = LocalDateTime.now().toInstant(ZoneOffset.of("+8")).toEpochMilli();
                MessageResultPojo messageResultPojo = new MessageResultPojo(command, countDownLatch, now, timeout, null);
                NettyMessageCache.NETTY_RESULT_TIMEOUT_MAP.put(key, messageResultPojo);
                NettyMessageCache.COMMAND_RESULT_SIGN_MAP.put(key, CrossInfoCache.getCrossInfoByIp(ip));
                sendMessage(ip, port, msg);
                countDownLatch.await(timeout, TimeUnit.MILLISECONDS);
                return NettyMessageCache.NETTY_RESULT_TIMEOUT_MAP.remove(key);
            }
        } catch (InterruptedException e) {
            log.error("sendMsg is error", e);
        }
        return null;
    }

    @Scheduled(fixedRate = 300)
    public void checkWaitTimeout() {
        long now = LocalDateTime.now().toInstant(ZoneOffset.of("+8")).toEpochMilli();
        NettyMessageCache.NETTY_RESULT_TIMEOUT_MAP.forEach((k, v) -> {
            commonThreadPoolExecutor.execute(() -> {
                if (now - v.getStartTime() > v.getWaitMillisecond()) {
                    log.warn("wait {}ms [{}] timeout", v.getWaitMillisecond(), k);
                    v.getCountDownLatch().countDown();
                }
            });
        });
    }
}