package net.wanji.utc.hisense.netty.handler;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.socket.DatagramPacket;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.spring.ServiceBeanContext;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.utc.hisense.cache.CrossInfoCache;
import net.wanji.utc.hisense.cache.SignalDataCache;
import net.wanji.utc.hisense.cache.netty.NettyMessageCache;
import net.wanji.utc.hisense.common.enums.CommandResultSignEnum;
import net.wanji.utc.hisense.common.enums.xml.MessageTypeEnum;
import net.wanji.utc.hisense.netty.commandsign.CommandResultSign;
import net.wanji.utc.hisense.netty.pojo.CommandPojo;
import net.wanji.utc.hisense.netty.response.CommandResponseFactory;
import net.wanji.utc.hisense.pojo.convert.RunningLightsStatusPojo;
import net.wanji.utc.hisense.pojo.netty.MessageResultPojo;
import net.wanji.utc.hisense.pojo.xml.pojo.messagecontent.HeartBeatRequest;
import net.wanji.utc.hisense.util.XMLUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2024/01/08 17:44
 */
@Slf4j
@Component
public class TcpClientHandler extends SimpleChannelInboundHandler<DatagramPacket> {

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("海信服务连接成功：{}", ctx.channel());
        ctx.fireChannelActive();
    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        log.error("收到消息channelRead:{}", msg);
        super.channelRead(ctx, msg);
    }

    /**
     * 读取消息
     *
     * @param ctx
     * @param packet
     * @throws Exception
     */
    @Override
    protected void channelRead0(ChannelHandlerContext ctx, DatagramPacket packet) throws Exception {
        log.error("收到消息channelRead0:{}", packet);
        //String result1 = result;
        //DatagramPacket packet = new DatagramPacket(null, null);
        ByteBuf content = packet.content();
        InetSocketAddress remote = packet.sender();
        int i = content.readableBytes();
        byte[] b = new byte[i];
        content.readBytes(b);

        String data = new String(b, StandardCharsets.UTF_8);
        Object resultPojo = CommandResultSign.getHexSign(data);
        log.error("当前收到海信数据转化实体： {}", resultPojo);
        String className = CommandResultSignEnum.getResultPojo(resultPojo);
        if (CommandResultSignEnum.GET_RUNNING_STATE_INFO.getClassName().equals(className)) {
            // 直接set灯态缓存
            RunningLightsStatusPojo pojo = (RunningLightsStatusPojo) resultPojo;
            String signalCode = pojo.getCID();
            CrossInfoPO crossInfoPO = CrossInfoCache.getCrossInfoBySignalCode(signalCode);
            if (Objects.nonNull(crossInfoPO)) {
                Map<String, RunningLightsStatusPojo> cache = SignalDataCache.runningStateInfoCacheUdp;
                cache.put(crossInfoPO.getId(), pojo);
            }
            return;
        }

        CommandResponseFactory commandResponseFactory = ServiceBeanContext.getBean(className);
        String key = StringUtils.join("/", remote.getHostString(), ":", remote.getPort(), "/", resultPojo.getClass().getSimpleName());
        CommandPojo commandPojo = getCommandPojo(key, resultPojo);
        Object hexResult = commandResponseFactory.getCommandResponse(commandPojo);
        if (Objects.nonNull(hexResult)) {
            setResultData(key, hexResult);
        }
    }

    private void setResultData(String key, Object hexResult) {
        MessageResultPojo resultPojo = NettyMessageCache.NETTY_RESULT_TIMEOUT_MAP.get(key);
        if (resultPojo != null) {
            resultPojo.setHexMessageResult(hexResult);
            resultPojo.getCountDownLatch().countDown();
        }
    }

    private CommandPojo getCommandPojo(String key, Object data) {
        CrossInfoPO crossInfo = NettyMessageCache.COMMAND_RESULT_SIGN_MAP.get(key);
        CommandPojo commandPojo = new CommandPojo();
        commandPojo.setCrossId(crossInfo.getId());
        commandPojo.setSignalId(crossInfo.getCode());
        commandPojo.setSignalIp(crossInfo.getIp());
        commandPojo.setPort(crossInfo.getPort());
        commandPojo.setMsg(data);
        return commandPojo;
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
    }

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        super.userEventTriggered(ctx, evt);
        if (evt instanceof IdleStateEvent) {
            IdleStateEvent idleStateEvent = (IdleStateEvent) evt;
            //向服务端发送消息
            try {
                SocketAddress socketAddress = ctx.channel().remoteAddress();
                log.info(socketAddress.toString());
                //向服务端发送心跳
                HeartBeatRequest heartBeatRequest = new HeartBeatRequest();
                heartBeatRequest.setMessageType(MessageTypeEnum.HEARTBEAT_STATUS.getType());
                ctx.channel().writeAndFlush(XMLUtils.strToHex(XMLUtils.convertToXml(heartBeatRequest)));
            } catch (Exception ex) {
                log.error("心跳消息发送失败：{}", ex.getMessage());
            }
        }

    }

}